use crate::components::achievement::AchievementComponent;
use crate::components::milestone::MilestoneComponent;
use yew::functional::*;
use yew::prelude::*;
use yew_router::prelude::*;

#[function_component]
pub fn Root() -> Html {
    let nav = use_navigator().expect("cannot get navigator");
    let app_state = use_context::<crate::AppState>().expect("no app state ctx found");

    let onclick_create_achievement = {
        let nav = nav.clone();
        Callback::from(move |_: web_sys::MouseEvent| {
            nav.push(&crate::Route::CreateAchievement);
        })
    };

    let onclick_create_milestone = Callback::from(move |_: web_sys::MouseEvent| {
        nav.push(&crate::Route::CreateMilestone);
    });

    let achievements = app_state
        .state
        .achievements
        .iter()
        .cloned()
        .map(|a| {
            html! {
                <AchievementComponent achievement={a} />
            }
        })
        .collect::<Html>();

    let completed_achievements = app_state
        .state
        .achievements
        .iter()
        .filter(|a| a.completed)
        .count();

    let mut milestones = app_state.state.milestones.clone();
    milestones.sort_by_key(|m| m.goal);
    let milestones = milestones.into_iter().map(|m| html! { <MilestoneComponent milestone={m} completed_achievements={completed_achievements} /> }).collect::<Html>();

    html! {
      <>

      <h1>{"Polter A"}<span style="color: var(--danger-color); text-decoration: line-through">{"bend"}</span>{"chievements!"}</h1>

      <hr />
      <div class="row flex flex-wrap">
        <div class="flex-grow">
          <h3>{"Milestones"}</h3>
        </div>

        <div class="flex-intrinsic-size">
          <button onclick={onclick_create_milestone} class="button-primary">{"New"}</button>
        </div>
      </div>
      {milestones}

      <hr />
      <div class="row flex flex-wrap">
        <div class="flex-grow">
          <h3>{"Achievements"}</h3>
        </div>

        <div class="flex-intrinsic-size">
          <button onclick={onclick_create_achievement} class="button-primary">{"New"}</button>
        </div>
      </div>
      {achievements}

      </>
    }
}
