use crate::event_bus::EventBus;
use crate::event_bus::EventBusInput;
use futures::channel::mpsc::Sender;
use futures::StreamExt;
use reqwasm::websocket::futures::WebSocket;
use reqwasm::websocket::Message;
use wasm_bindgen_futures::spawn_local;
use yew_agent::Dispatched;

#[derive(Debug)]
pub struct WebsocketService {
    // No messages sent from app to server on websocket at the moment.
    pub tx: Sender<()>,
}

impl WebsocketService {
    pub fn connect() -> Self {
        let hostname = web_sys::window().unwrap().location().hostname().unwrap();
        let ws_addr = format!("ws://{hostname}:4000/ws");
        let ws = WebSocket::open(&ws_addr).unwrap();
        log::info!("Opened websocket connection to {ws_addr}");

        let (_write, mut read) = ws.split();

        let (in_tx, mut in_rx) = futures::channel::mpsc::channel::<()>(1000);
        let mut event_bus = EventBus::<common::WebSocketMessage>::dispatcher();

        // App to Server
        spawn_local(async move {
            while let Some(()) = in_rx.next().await {
                // write.send(Message::Text(s)).await.unwrap();
            }
        });

        // Server to App
        spawn_local(async move {
            while let Some(msg) = read.next().await {
                match msg {
                    Ok(Message::Text(data)) => {
                        match serde_json::from_str::<common::WebSocketMessage>(&data) {
                            Ok(ws_msg) => {
                                log::debug!("Received ws message. Dispatching to event bus.");
                                event_bus.send(EventBusInput::EventBusMsg(ws_msg));
                            }
                            Err(err) => {
                                log::error!("{err:?}");
                            }
                        }
                    }
                    Ok(Message::Bytes(_)) => {
                        log::warn!("Received binary data on websocket. This is unhandled");
                    }
                    Err(err) => {
                        log::error!("ws error: {err:?}");
                    }
                }
            }
            log::debug!("WebSocket Closed");
        });

        Self { tx: in_tx }
    }
}
